import * as v from '../../shared/src/valita.ts';
import type { Read, Write } from './dag/store.ts';
import type { ClientGroupID, ClientID } from './sync/ids.ts';
/**
 * We keep track of deleted clients in the {@linkcode DELETED_CLIENTS_HEAD_NAME}
 * head.
 */
export declare const DELETED_CLIENTS_HEAD_NAME = "deleted-clients";
export declare const deletedClientsSchema: v.ObjectType<Readonly<{
    clientIDs: v.Type<readonly string[]>;
    clientGroupIDs: v.Type<readonly string[]>;
}>, undefined>;
export type DeletedClients = v.Infer<typeof deletedClientsSchema>;
export declare function setDeletedClients(dagWrite: Write, clientIDs: readonly ClientID[], clientGroupIDs: readonly ClientGroupID[]): Promise<DeletedClients>;
export declare function getDeletedClients(dagRead: Read): Promise<DeletedClients>;
/**
 * Adds deleted clients to the {@linkcode DELETED_CLIENTS_HEAD_NAME} head.
 * @returns the new list of deleted clients (sorted and deduped).
 */
export declare function addDeletedClients(dagWrite: Write, clientIDs: ClientID[], clientGroupIDs: ClientGroupID[]): Promise<DeletedClients>;
export declare function removeDeletedClients(dagWrite: Write, clientIDs: readonly ClientID[], clientGroupIDs: readonly ClientGroupID[]): Promise<DeletedClients>;
/**
 * Sorts and dedupes the given array.
 */
export declare function normalize<T>(arr: readonly T[]): T[];
//# sourceMappingURL=deleted-clients.d.ts.map