type GlobalThis = typeof globalThis;
export declare function overrideBrowserGlobal<T extends keyof GlobalThis>(name: T, value: GlobalThis[T]): void;
export declare function clearBrowserOverrides(): void;
export declare function getBrowserGlobal<T extends keyof GlobalThis>(name: T): GlobalThis[T] | undefined;
/**
 * Returns the global method with the given name, bound to the global object.
 * This is important because some methods (e.g. `requestAnimationFrame`) are not
 * bound to the global object by default.
 *
 * If you end up using {@linkcode getBrowserGlobal} instead in a case like this:
 *
 * ```js
 * this.#raf = getBrowserGlobal('requestAnimationFrame') ?? rafFallback;
 * ...
 * this.#raf(() => ...);
 * ```
 *
 * You will end up with `Uncaught TypeError: Illegal invocation` because `this`
 * is not bound to the global object
 */
export declare function getBrowserGlobalMethod<T extends keyof GlobalThis>(name: T): GlobalThis[T] | undefined;
export declare function mustGetBrowserGlobal<T extends keyof GlobalThis>(name: T): GlobalThis[T];
export {};
//# sourceMappingURL=browser-env.d.ts.map