interface ReadonlySetLike<T> {
    /**
     * Despite its name, returns an iterator of the values in the set-like.
     */
    keys(): Iterator<T>;
    /**
     * @returns a boolean indicating whether an element with the specified value exists in the set-like or not.
     */
    has(value: T): boolean;
    /**
     * @returns the number of (unique) elements in the set-like.
     */
    readonly size: number;
}
export declare function equals<T>(a: ReadonlySetLike<T>, b: ReadonlySetLike<T>): boolean;
export declare function union<T>(...sets: ReadonlySetLike<T>[]): Set<T>;
export declare function intersection<T>(a: ReadonlySetLike<T>, b: ReadonlySetLike<T>): Set<T>;
/**
 * Returns the elements in {@linkcode a} that are not in {@linkcode b}.
 */
export declare function difference<T>(a: ReadonlySetLike<T>, b: ReadonlySetLike<T>): Set<T>;
export declare function symmetricDifferences<T>(a: ReadonlySetLike<T>, b: ReadonlySetLike<T>): [onlyA: Set<T>, onlyB: Set<T>];
export {};
//# sourceMappingURL=set-utils.d.ts.map