/**
 * These types represent the _compiled_ config whereas `define-config` types represent the _source_ config.
 */
import { type Config } from '../../../shared/src/options.ts';
import * as v from '../../../shared/src/valita.ts';
export type { LogConfig } from '../../../otel/src/log-options.ts';
export declare const appOptions: {
    id: {
        type: v.Type<string>;
        desc: string[];
    };
    publications: {
        type: v.Type<string[]>;
        desc: string[];
    };
};
export declare const shardOptions: {
    id: {
        type: v.Optional<string>;
        hidden: boolean;
    };
    num: {
        type: v.Type<number>;
        desc: string[];
        hidden: boolean;
    };
};
declare const replicaOptions: {
    file: {
        type: v.Type<string>;
        desc: string[];
    };
    vacuumIntervalHours: {
        type: v.Optional<number>;
        desc: string[];
    };
};
export type ReplicaOptions = Config<typeof replicaOptions>;
declare const perUserMutationLimit: {
    max: {
        type: v.Optional<number>;
        desc: string[];
    };
    windowMs: {
        type: v.Type<number>;
        desc: string[];
    };
};
export type RateLimit = Config<typeof perUserMutationLimit>;
declare const authOptions: {
    jwk: {
        type: v.Optional<string>;
        desc: string[];
    };
    jwksUrl: {
        type: v.Optional<string>;
        desc: string[];
    };
    secret: {
        type: v.Optional<string>;
        desc: string[];
    };
};
export type AuthConfig = Config<typeof authOptions>;
export declare const zeroOptions: {
    upstream: {
        db: {
            type: v.Type<string>;
            desc: string[];
        };
        type: {
            type: v.Type<"pg" | "custom">;
            desc: string[];
            hidden: boolean;
        };
        maxConns: {
            type: v.Type<number>;
            desc: string[];
        };
        maxConnsPerWorker: {
            type: v.Optional<number>;
            hidden: boolean;
        };
    };
    push: {
        url: {
            type: v.Optional<string>;
            desc: string[];
        };
        apiKey: {
            type: v.Optional<string>;
            desc: string[];
        };
    };
    cvr: {
        db: {
            type: v.Optional<string>;
            desc: string[];
        };
        maxConns: {
            type: v.Type<number>;
            desc: string[];
        };
        maxConnsPerWorker: {
            type: v.Optional<number>;
            hidden: boolean;
        };
    };
    queryHydrationStats: {
        type: v.Optional<boolean>;
        desc: string[];
    };
    change: {
        db: {
            type: v.Optional<string>;
            desc: string[];
        };
        maxConns: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    replica: {
        file: {
            type: v.Type<string>;
            desc: string[];
        };
        vacuumIntervalHours: {
            type: v.Optional<number>;
            desc: string[];
        };
    };
    log: {
        level: v.Type<"error" | "debug" | "info" | "warn">;
        format: {
            type: v.Type<"text" | "json">;
            desc: string[];
        };
        slowRowThreshold: {
            type: v.Type<number>;
            desc: string[];
        };
        slowHydrateThreshold: {
            type: v.Type<number>;
            desc: string[];
        };
        ivmSampling: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    app: {
        id: {
            type: v.Type<string>;
            desc: string[];
        };
        publications: {
            type: v.Type<string[]>;
            desc: string[];
        };
    };
    shard: {
        id: {
            type: v.Optional<string>;
            hidden: boolean;
        };
        num: {
            type: v.Type<number>;
            desc: string[];
            hidden: boolean;
        };
    };
    auth: {
        jwk: {
            type: v.Optional<string>;
            desc: string[];
        };
        jwksUrl: {
            type: v.Optional<string>;
            desc: string[];
        };
        secret: {
            type: v.Optional<string>;
            desc: string[];
        };
    };
    port: {
        type: v.Type<number>;
        desc: string[];
    };
    changeStreamer: {
        mode: {
            type: v.Type<"dedicated" | "discover">;
            desc: string[];
        };
        port: {
            type: v.Optional<number>;
            desc: string[];
        };
        address: {
            type: v.Optional<string>;
            desc: string[];
        };
        discoveryInterfacePreferences: {
            type: v.Type<string[]>;
            desc: string[];
            hidden: boolean;
        };
        uri: {
            type: v.Optional<string>;
            hidden: boolean;
        };
    };
    taskID: {
        type: v.Optional<string>;
        desc: string[];
    };
    perUserMutationLimit: {
        max: {
            type: v.Optional<number>;
            desc: string[];
        };
        windowMs: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    numSyncWorkers: {
        type: v.Optional<number>;
        desc: string[];
    };
    autoReset: {
        type: v.Type<boolean>;
        desc: string[];
    };
    adminPassword: {
        type: v.Optional<string>;
        desc: string[];
    };
    litestream: {
        executable: {
            type: v.Optional<string>;
            desc: string[];
        };
        configPath: {
            type: v.Type<string>;
            desc: string[];
        };
        logLevel: {
            type: v.Type<"error" | "debug" | "info" | "warn">;
        };
        backupURL: {
            type: v.Optional<string>;
            desc: string[];
        };
        port: {
            type: v.Optional<number>;
            desc: string[];
        };
        checkpointThresholdMB: {
            type: v.Type<number>;
            desc: string[];
        };
        incrementalBackupIntervalMinutes: {
            type: v.Type<number>;
            desc: string[];
        };
        snapshotBackupIntervalHours: {
            type: v.Type<number>;
            desc: string[];
        };
        restoreParallelism: {
            type: v.Type<number>;
            desc: string[];
        };
        multipartConcurrency: {
            type: v.Type<number>;
            desc: string[];
        };
        multipartSize: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    storageDBTmpDir: {
        type: v.Optional<string>;
        desc: string[];
    };
    initialSync: {
        tableCopyWorkers: {
            type: v.Type<number>;
            desc: string[];
        };
        rowBatchSize: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    tenantID: {
        type: v.Optional<string>;
        desc: string[];
        hidden: boolean;
    };
    targetClientRowCount: {
        type: v.Type<number>;
        desc: string[];
    };
    lazyStartup: {
        type: v.Type<boolean>;
        desc: string[];
    };
    serverVersion: {
        type: v.Optional<string>;
        desc: string[];
    };
};
export type ZeroConfig = Config<typeof zeroOptions>;
export declare const ZERO_ENV_VAR_PREFIX = "ZERO_";
export declare function getZeroConfig(env?: NodeJS.ProcessEnv, argv?: string[]): Config<{
    upstream: {
        db: {
            type: v.Type<string>;
            desc: string[];
        };
        type: {
            type: v.Type<"pg" | "custom">;
            desc: string[];
            hidden: boolean;
        };
        maxConns: {
            type: v.Type<number>;
            desc: string[];
        };
        maxConnsPerWorker: {
            type: v.Optional<number>;
            hidden: boolean;
        };
    };
    push: {
        url: {
            type: v.Optional<string>;
            desc: string[];
        };
        apiKey: {
            type: v.Optional<string>;
            desc: string[];
        };
    };
    cvr: {
        db: {
            type: v.Optional<string>;
            desc: string[];
        };
        maxConns: {
            type: v.Type<number>;
            desc: string[];
        };
        maxConnsPerWorker: {
            type: v.Optional<number>;
            hidden: boolean;
        };
    };
    queryHydrationStats: {
        type: v.Optional<boolean>;
        desc: string[];
    };
    change: {
        db: {
            type: v.Optional<string>;
            desc: string[];
        };
        maxConns: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    replica: {
        file: {
            type: v.Type<string>;
            desc: string[];
        };
        vacuumIntervalHours: {
            type: v.Optional<number>;
            desc: string[];
        };
    };
    log: {
        level: v.Type<"error" | "debug" | "info" | "warn">;
        format: {
            type: v.Type<"text" | "json">;
            desc: string[];
        };
        slowRowThreshold: {
            type: v.Type<number>;
            desc: string[];
        };
        slowHydrateThreshold: {
            type: v.Type<number>;
            desc: string[];
        };
        ivmSampling: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    app: {
        id: {
            type: v.Type<string>;
            desc: string[];
        };
        publications: {
            type: v.Type<string[]>;
            desc: string[];
        };
    };
    shard: {
        id: {
            type: v.Optional<string>;
            hidden: boolean;
        };
        num: {
            type: v.Type<number>;
            desc: string[];
            hidden: boolean;
        };
    };
    auth: {
        jwk: {
            type: v.Optional<string>;
            desc: string[];
        };
        jwksUrl: {
            type: v.Optional<string>;
            desc: string[];
        };
        secret: {
            type: v.Optional<string>;
            desc: string[];
        };
    };
    port: {
        type: v.Type<number>;
        desc: string[];
    };
    changeStreamer: {
        mode: {
            type: v.Type<"dedicated" | "discover">;
            desc: string[];
        };
        port: {
            type: v.Optional<number>;
            desc: string[];
        };
        address: {
            type: v.Optional<string>;
            desc: string[];
        };
        discoveryInterfacePreferences: {
            type: v.Type<string[]>;
            desc: string[];
            hidden: boolean;
        };
        uri: {
            type: v.Optional<string>;
            hidden: boolean;
        };
    };
    taskID: {
        type: v.Optional<string>;
        desc: string[];
    };
    perUserMutationLimit: {
        max: {
            type: v.Optional<number>;
            desc: string[];
        };
        windowMs: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    numSyncWorkers: {
        type: v.Optional<number>;
        desc: string[];
    };
    autoReset: {
        type: v.Type<boolean>;
        desc: string[];
    };
    adminPassword: {
        type: v.Optional<string>;
        desc: string[];
    };
    litestream: {
        executable: {
            type: v.Optional<string>;
            desc: string[];
        };
        configPath: {
            type: v.Type<string>;
            desc: string[];
        };
        logLevel: {
            type: v.Type<"error" | "debug" | "info" | "warn">;
        };
        backupURL: {
            type: v.Optional<string>;
            desc: string[];
        };
        port: {
            type: v.Optional<number>;
            desc: string[];
        };
        checkpointThresholdMB: {
            type: v.Type<number>;
            desc: string[];
        };
        incrementalBackupIntervalMinutes: {
            type: v.Type<number>;
            desc: string[];
        };
        snapshotBackupIntervalHours: {
            type: v.Type<number>;
            desc: string[];
        };
        restoreParallelism: {
            type: v.Type<number>;
            desc: string[];
        };
        multipartConcurrency: {
            type: v.Type<number>;
            desc: string[];
        };
        multipartSize: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    storageDBTmpDir: {
        type: v.Optional<string>;
        desc: string[];
    };
    initialSync: {
        tableCopyWorkers: {
            type: v.Type<number>;
            desc: string[];
        };
        rowBatchSize: {
            type: v.Type<number>;
            desc: string[];
        };
    };
    tenantID: {
        type: v.Optional<string>;
        desc: string[];
        hidden: boolean;
    };
    targetClientRowCount: {
        type: v.Type<number>;
        desc: string[];
    };
    lazyStartup: {
        type: v.Type<boolean>;
        desc: string[];
    };
    serverVersion: {
        type: v.Optional<string>;
        desc: string[];
    };
}>;
//# sourceMappingURL=zero-config.d.ts.map