import type { LogContext } from '@rocicorp/logger';
import type { Database } from '../../../zqlite/src/db.ts';
import type { LiteAndZqlSpec, LiteIndexSpec, LiteTableSpec } from './specs.ts';
export declare function listTables(db: Database): LiteTableSpec[];
export declare function listIndexes(db: Database): LiteIndexSpec[];
/**
 * Computes a TableSpec "view" of the replicated data that is
 * suitable for processing / consumption for the client. This
 * includes:
 * * excluding tables without a PRIMARY KEY or UNIQUE INDEX
 * * excluding columns with types that are not supported by ZQL
 * * choosing columns to use as the primary key amongst those
 *   in unique indexes
 *
 * @param tableSpecs an optional map to reset and populate
 * @param fullTables an optional map to receive the full table specs,
 *        which may include tables and columns that are not synced to
 *        the client because they lack a primary key or are of unsupported
 *        data types.
 */
export declare function computeZqlSpecs(lc: LogContext, replica: Database, tableSpecs?: Map<string, LiteAndZqlSpec>, fullTables?: Map<string, LiteTableSpec>): Map<string, LiteAndZqlSpec>;
//# sourceMappingURL=lite-tables.d.ts.map