import { logs, SeverityNumber, } from '@opentelemetry/api-logs';
import { errorOrObject } from "./logging.js";
import { stringify } from "../types/bigint-json.js";
import { startOtelAuto } from "./otel-start.js";
export class OtelLogSink {
    #logger;
    constructor() {
        // start otel in case it was not started yet
        // this is a no-op if already started
        startOtelAuto();
        this.#logger = logs.getLogger('zero-cache');
    }
    log(level, context, ...args) {
        const lastObj = errorOrObject(args.at(-1));
        if (lastObj) {
            args.pop();
        }
        let message = args.length
            ? args.map(s => (typeof s === 'string' ? s : stringify(s))).join(' ')
            : '';
        if (lastObj) {
            message += ` ${stringify(lastObj)}`;
        }
        const payload = {
            severityText: level,
            severityNumber: toErrorNum(level),
            body: message,
        };
        if (context) {
            payload.attributes = context;
        }
        this.#logger.emit(payload);
    }
}
function toErrorNum(level) {
    switch (level) {
        case 'error':
            return SeverityNumber.ERROR;
        case 'warn':
            return SeverityNumber.WARN;
        case 'info':
            return SeverityNumber.INFO;
        case 'debug':
            return SeverityNumber.DEBUG;
        default:
            throw new Error(`Unknown log level: ${level}`);
    }
}
//# sourceMappingURL=otel-log-sink.js.map