import type { LogContext } from '@rocicorp/logger';
import * as v from '../../../../../../shared/src/valita.ts';
import type { PostgresDB, PostgresTransaction } from '../../../../types/pg.ts';
import type { AppID, ShardConfig, ShardID } from '../../../../types/shards.ts';
import { type PublicationInfo, type PublishedSchema } from './published.ts';
export declare function internalPublicationPrefix({ appID }: AppID): string;
export declare function legacyReplicationSlot({ appID, shardNum }: ShardID): string;
export declare function replicationSlotPrefix(shard: ShardID): string;
/**
 * An expression used to match replication slots in the shard
 * in a Postgres `LIKE` operator.
 */
export declare function replicationSlotExpression(shard: ShardID): string;
export declare function newReplicationSlot(shard: ShardID): string;
export declare function ensureGlobalTables(db: PostgresDB, appID: AppID): Promise<void>;
export declare function getClientsTableDefinition(schema: string): string;
export declare const SHARD_CONFIG_TABLE = "shardConfig";
export declare function shardSetup(shardConfig: ShardConfig, metadataPublication: string): string;
export declare function dropShard(appID: string, shardID: string | number): string;
declare const internalShardConfigSchema: v.ObjectType<{
    publications: v.ArrayType<v.Type<string>>;
    ddlDetection: v.Type<boolean>;
}, undefined>;
export type InternalShardConfig = v.Infer<typeof internalShardConfigSchema>;
declare const replicaSchema: v.ObjectType<Omit<{
    publications: v.ArrayType<v.Type<string>>;
    ddlDetection: v.Type<boolean>;
}, "version" | "slot" | "initialSchema"> & {
    slot: v.Type<string>;
    version: v.Type<string>;
    initialSchema: v.ObjectType<Omit<{
        tables: v.ArrayType<v.ObjectType<Omit<Omit<{
            name: v.Type<string>;
            columns: v.Type<Record<string, {
                pgTypeClass?: "b" | "e" | "d" | "c" | "p" | "r" | "m" | undefined;
                characterMaximumLength?: number | null | undefined;
                notNull?: boolean | null | undefined;
                dflt?: string | null | undefined;
                pos: number;
                dataType: string;
            }>>;
            primaryKey: v.Optional<string[]>;
        }, "schema"> & {
            schema: v.Type<string>;
        }, "publications" | "columns" | "oid" | "replicaIdentity"> & {
            oid: v.Type<number>;
            columns: v.Type<Record<string, {
                pgTypeClass?: "b" | "e" | "d" | "c" | "p" | "r" | "m" | undefined;
                characterMaximumLength?: number | null | undefined;
                notNull?: boolean | null | undefined;
                dflt?: string | null | undefined;
                pos: number;
                dataType: string;
                typeOID: number;
            }>>;
            replicaIdentity: v.Optional<"n" | "d" | "f" | "i">;
            publications: v.Type<Record<string, {
                rowFilter: string | null;
            }>>;
        }, undefined>>;
    }, "indexes"> & {
        indexes: v.ArrayType<v.ObjectType<Omit<Omit<{
            name: v.Type<string>;
            tableName: v.Type<string>;
            unique: v.Type<boolean>;
            columns: v.Type<Record<string, "ASC" | "DESC">>;
        }, "schema"> & {
            schema: v.Type<string>;
        }, "isReplicaIdentity" | "isImmediate"> & {
            isReplicaIdentity: v.Optional<boolean>;
            isImmediate: v.Optional<boolean>;
        }, undefined>>;
    }, undefined>;
}, undefined>;
export type Replica = v.Infer<typeof replicaSchema>;
export declare function addReplica(sql: PostgresDB, shard: ShardID, slot: string, replicaVersion: string, { tables, indexes }: PublishedSchema): Promise<void>;
export declare function getReplicaAtVersion(sql: PostgresDB, shard: ShardID, replicaVersion: string): Promise<Replica | null>;
export declare function getInternalShardConfig(sql: PostgresDB, shard: ShardID): Promise<InternalShardConfig>;
/**
 * Sets up and returns all publications (including internal ones) for
 * the given shard.
 */
export declare function setupTablesAndReplication(lc: LogContext, sql: PostgresTransaction, requested: ShardConfig): Promise<void>;
export declare function setupTriggers(lc: LogContext, tx: PostgresTransaction, shard: ShardConfig): Promise<void>;
export declare function validatePublications(lc: LogContext, published: PublicationInfo): void;
type ReplicaIdentities = {
    apply(lc: LogContext, db: PostgresDB): Promise<void>;
};
export declare function replicaIdentitiesForTablesWithoutPrimaryKeys(pubs: PublishedSchema): ReplicaIdentities | undefined;
export {};
//# sourceMappingURL=shard.d.ts.map