import { type Result, Subscription } from '../../types/subscription.ts';
import type { ReplicaState, ReplicaStateNotifier } from './replicator.ts';
/**
 * Handles the semantics of {@link ReplicatorVersionNotifier.subscribe()}
 * notifications, namely:
 *
 * * New subscribers are notified immediately with the latest received
 *   ReplicaState.
 *
 * * Non-latest notifications are discarded if the subscriber is too
 *   busy to consume them.
 *
 * By encapsulating the state for the first behavior (essentially, whether
 * the first notification has been sent by the Replicator), Notifier objects
 * can be chained to simplify fanout from Replicator to View Syncers.
 *
 * In particular, each Syncer Thread can manage a single Subscription to
 * the Replicator across a MessageChannel, which it uses for its own Notifier
 * instance to manage subscriptions from View Syncers within its thread. This
 * way a Replicator only deals with sending notifications to a bounded set
 * of MessageChannel-based subscribers (Syncer Threads), while the dynamic
 * subscribe and unsubscribe traffic from View Syncers remains within each
 * Syncer Thread.
 */
export declare class Notifier implements ReplicaStateNotifier {
    #private;
    subscribe(): Subscription<ReplicaState>;
    notifySubscribers(state?: ReplicaState): Promise<Result>[];
}
//# sourceMappingURL=notifier.d.ts.map