import type { NoIndexDiff } from '../../../replicache/src/btree/node.ts';
import type { Hash } from '../../../replicache/src/hash.ts';
import type { AST } from '../../../zero-protocol/src/ast.ts';
import type { FilterInput } from '../../../zql/src/ivm/filter-operators.ts';
import type { Input, Storage } from '../../../zql/src/ivm/operator.ts';
import type { Source } from '../../../zql/src/ivm/source.ts';
import type { CommitListener, GotCallback, QueryDelegate } from '../../../zql/src/query/query-impl.ts';
import type { RunOptions } from '../../../zql/src/query/query.ts';
import type { TTL } from '../../../zql/src/query/ttl.ts';
import { type IVMSourceBranch } from './ivm-branch.ts';
import type { QueryManager } from './query-manager.ts';
import type { ZeroLogContext } from './zero-log-context.ts';
export type AddQuery = QueryManager['add'];
export type UpdateQuery = QueryManager['update'];
/**
 * ZeroContext glues together zql and Replicache. It listens to changes in
 * Replicache data and pushes them into IVM and on tells the server about new
 * queries.
 */
export declare class ZeroContext implements QueryDelegate {
    #private;
    readonly assertValidRunOptions: (options?: RunOptions) => void;
    /**
     * Client-side queries start out as "unknown" and are then updated to
     * "complete" once the server has sent back the query result.
     */
    readonly defaultQueryComplete = false;
    constructor(lc: ZeroLogContext, mainSources: IVMSourceBranch, addQuery: AddQuery, updateQuery: UpdateQuery, batchViewUpdates: (applyViewUpdates: () => void) => void, slowMaterializeThreshold: number, assertValidRunOptions: (options?: RunOptions) => void);
    getSource(name: string): Source | undefined;
    addServerQuery(ast: AST, ttl: TTL, gotCallback?: GotCallback | undefined): () => void;
    updateServerQuery(ast: AST, ttl: TTL): void;
    onQueryMaterialized(hash: string, ast: AST, duration: number): void;
    mapAst(ast: AST): AST;
    createStorage(): Storage;
    decorateInput(input: Input): Input;
    decorateFilterInput(input: FilterInput): FilterInput;
    onTransactionCommit(cb: CommitListener): () => void;
    batchViewUpdates<T>(applyViewUpdates: () => T): T;
    processChanges(expectedHead: Hash | undefined, newHead: Hash, changes: NoIndexDiff): void;
}
//# sourceMappingURL=context.d.ts.map