import type { EphemeralID, MutationTrackingData } from '../../../replicache/src/replicache-options.ts';
import type { PushResponse } from '../../../zero-protocol/src/push.ts';
import type { ZeroLogContext } from './zero-log-context.ts';
/**
 * Tracks what pushes are in-flight and resolves promises when they're acked.
 */
export declare class MutationTracker {
    #private;
    constructor(lc: ZeroLogContext);
    set clientID(clientID: string);
    trackMutation(): MutationTrackingData;
    mutationIDAssigned(id: EphemeralID, mutationID: number): void;
    /**
     * Reject the mutation due to an unhandled exception on the client.
     * The mutation must not have been persisted to the client store.
     */
    rejectMutation(id: EphemeralID, e: unknown): void;
    processPushResponse(response: PushResponse): void;
    /**
     * When we reconnect to zero-cache, we resolve all outstanding mutations
     * whose ID is less than or equal to the lastMutationID.
     *
     * The reason is that any responses the API server sent
     * to those mutations have been lost.
     *
     * An example case: the API server responds while the connection
     * is down. Those responses are dropped.
     *
     * Mutations whose LMID is > the lastMutationID are not resolved
     * since they will be retried by the client, giving us another chance
     * at getting a response.
     *
     * The only way to ensure that all API server responses are
     * received would be to have the API server write them
     * to the DB while writing the LMID.
     *
     * This would have the downside of not being able to provide responses to a
     * mutation with data gathered after the transaction.
     */
    onConnected(lastMutationID: number): void;
    get size(): number;
    onAllMutationsConfirmed(listener: () => void): void;
}
//# sourceMappingURL=mutation-tracker.d.ts.map