import type { JSONValue } from '../../shared/src/json.ts';
import type { DBConnection, DBTransaction, Row } from '../../zql/src/mutate/custom.ts';
/**
 * Subset of the postgres lib's `Transaction` interface that we use.
 */
export type PostgresJSTransaction = {
    unsafe(sql: string, params: JSONValue[]): Promise<Row[]>;
};
/**
 * Subset of the postgres lib's `SQL` interface that we use.
 */
export type PostgresJSClient<Transaction extends PostgresJSTransaction> = {
    unsafe(sql: string, params: JSONValue[]): Promise<Row[]>;
    begin<T>(fn: (tx: Transaction) => Promise<T>): Promise<T>;
};
/**
 * Implements the `DBConnection` interface needed by PushProcessor for the
 * postgres.js library.
 */
export declare class PostgresJSConnection<WrappedTransaction extends PostgresJSTransaction, WrappedPostgres extends PostgresJSClient<WrappedTransaction>> implements DBConnection<WrappedTransaction> {
    #private;
    constructor(pg: WrappedPostgres);
    query(sql: string, params: unknown[]): Promise<Row[]>;
    transaction<T>(fn: (tx: DBTransaction<WrappedTransaction>) => Promise<T>): Promise<T>;
}
//# sourceMappingURL=postgresjs-connection.d.ts.map