import { formatPg, sql } from "../../z2s/src/sql.js";
import { makeSchemaQuery } from "./query.js";
import { makeSchemaCRUD, TransactionImpl } from "./custom.js";
import { makeServerTransaction } from "./custom.js";
/**
 * Implements a Database for use with PushProcessor that is backed by Postgres.
 *
 * This implementation also implements the same ZQL interfaces for reading and
 * writing data that the Zero client does, so that mutator functions can be
 * shared across client and server.
 */
export class ZQLDatabase {
    #connection;
    #mutate;
    #query;
    #schema;
    constructor(connection, schema) {
        this.#connection = connection;
        this.#mutate = makeSchemaCRUD(schema);
        this.#query = makeSchemaQuery(schema);
        this.#schema = schema;
    }
    transaction(callback, transactionInput) {
        return this.#connection.transaction(async (dbTx) => {
            const zeroTx = await makeServerTransaction(dbTx, transactionInput.clientID, transactionInput.mutationID, this.#schema, this.#mutate, this.#query);
            return callback(zeroTx, {
                async updateClientMutationID() {
                    const formatted = formatPg(sql `INSERT INTO ${sql.ident(transactionInput.upstreamSchema)}.clients 
                    as current ("clientGroupID", "clientID", "lastMutationID")
                        VALUES (${transactionInput.clientGroupID}, ${transactionInput.clientID}, ${1})
                    ON CONFLICT ("clientGroupID", "clientID")
                    DO UPDATE SET "lastMutationID" = current."lastMutationID" + 1
                    RETURNING "lastMutationID"`);
                    const [{ lastMutationID }] = (await dbTx.query(formatted.text, formatted.values));
                    return { lastMutationID };
                },
            });
        });
    }
}
//# sourceMappingURL=zql-database.js.map