import { mapAllEntries } from "../../shared/src/objects.js";
import * as v from "../../shared/src/valita.js";
export const valueTypeSchema = v.literalUnion('string', 'number', 'boolean', 'null', 'json');
export const columnSchemaSchema = v.object({
    type: valueTypeSchema,
});
export const tableSchemaSchema = v.object({
    columns: v.record(columnSchemaSchema),
});
export const clientSchemaSchema = v.object({
    tables: v.record(tableSchemaSchema),
});
const keyCmp = ([a], [b]) => a < b ? -1 : a > b ? 1 : 0;
/**
 * Returns a normalized schema (with the tables and columns sorted)
 * suitable for hashing.
 */
export function normalizeClientSchema(schema) {
    return {
        tables: mapAllEntries(schema.tables, tables => tables
            .sort(keyCmp)
            .map(([name, table]) => [
            name,
            { columns: mapAllEntries(table.columns, e => e.sort(keyCmp)) },
        ])),
    };
}
//# sourceMappingURL=client-schema.js.map