import * as v from '../../shared/src/valita.ts';
/**
 * After opening a websocket the client waits for a `connected` message
 * from the server.  It then sends an `initConnection` message to the
 * server.  The server waits for the `initConnection` message before
 * beginning to send pokes to the newly connected client, so as to avoid
 * syncing lots of queries which are no longer desired by the client.
 */
export declare const connectedBodySchema: v.ObjectType<{
    wsid: v.Type<string>;
    timestamp: v.Optional<number>;
}, undefined>;
export declare const connectedMessageSchema: v.TupleType<[v.Type<"connected">, v.ObjectType<{
    wsid: v.Type<string>;
    timestamp: v.Optional<number>;
}, undefined>]>;
declare const userPushParamsSchema: v.ObjectType<{
    queryParams: v.Optional<Record<string, string>>;
}, undefined>;
declare const initConnectionBodySchema: v.ObjectType<{
    desiredQueriesPatch: v.ArrayType<v.UnionType<[v.ObjectType<{
        op: v.Type<"put">;
        hash: v.Type<string>;
        ast: v.Type<import("./ast.ts").AST>;
        ttl: v.Optional<number>;
    }, undefined>, v.ObjectType<{
        op: v.Type<"del">;
        hash: v.Type<string>;
    }, undefined>, v.ObjectType<{
        op: v.Type<"clear">;
    }, undefined>]>>;
    clientSchema: v.Optional<{
        tables: Record<string, {
            columns: Record<string, {
                type: import("./client-schema.ts").ValueType;
            }>;
        }>;
    }>;
    deleted: v.Optional<{
        readonly clientIDs?: readonly string[] | undefined;
        readonly clientGroupIDs?: readonly string[] | undefined;
    }>;
    userPushParams: v.Optional<{
        queryParams?: Record<string, string> | undefined;
    }>;
}, undefined>;
export declare const initConnectionMessageSchema: v.TupleType<[v.Type<"initConnection">, v.ObjectType<{
    desiredQueriesPatch: v.ArrayType<v.UnionType<[v.ObjectType<{
        op: v.Type<"put">;
        hash: v.Type<string>;
        ast: v.Type<import("./ast.ts").AST>;
        ttl: v.Optional<number>;
    }, undefined>, v.ObjectType<{
        op: v.Type<"del">;
        hash: v.Type<string>;
    }, undefined>, v.ObjectType<{
        op: v.Type<"clear">;
    }, undefined>]>>;
    clientSchema: v.Optional<{
        tables: Record<string, {
            columns: Record<string, {
                type: import("./client-schema.ts").ValueType;
            }>;
        }>;
    }>;
    deleted: v.Optional<{
        readonly clientIDs?: readonly string[] | undefined;
        readonly clientGroupIDs?: readonly string[] | undefined;
    }>;
    userPushParams: v.Optional<{
        queryParams?: Record<string, string> | undefined;
    }>;
}, undefined>]>;
export type ConnectedBody = v.Infer<typeof connectedBodySchema>;
export type ConnectedMessage = v.Infer<typeof connectedMessageSchema>;
export type UserPushParams = v.Infer<typeof userPushParamsSchema>;
export type InitConnectionBody = v.Infer<typeof initConnectionBodySchema>;
export type InitConnectionMessage = v.Infer<typeof initConnectionMessageSchema>;
export declare function encodeSecProtocols(initConnectionMessage: InitConnectionMessage | undefined, authToken: string | undefined): string;
export declare function decodeSecProtocols(secProtocol: string): {
    initConnectionMessage: InitConnectionMessage | undefined;
    authToken: string | undefined;
};
export {};
//# sourceMappingURL=connect.d.ts.map