import { unreachable } from "../../../shared/src/asserts.js";
/**
 * Catch is an Output that collects all incoming stream data into arrays. Mainly
 * useful for testing.
 */
export class Catch {
    #input;
    #fetchOnPush;
    pushes = [];
    pushesWithFetch = [];
    constructor(input, fetchOnPush = false) {
        this.#input = input;
        this.#fetchOnPush = fetchOnPush;
        input.setOutput(this);
    }
    fetch(req = {}) {
        return [...this.#input.fetch(req)].map(expandNode);
    }
    cleanup(req = {}) {
        return [...this.#input.cleanup(req)].map(expandNode);
    }
    push(change) {
        const fetch = this.#fetchOnPush
            ? [...this.#input.fetch({})].map(expandNode)
            : [];
        const expandedChange = expandChange(change);
        if (this.#fetchOnPush) {
            this.pushesWithFetch.push({
                change: expandedChange,
                fetch,
            });
        }
        this.pushes.push(expandedChange);
    }
    reset() {
        this.pushes.length = 0;
    }
    destroy() {
        this.#input.destroy();
    }
}
export function expandChange(change) {
    switch (change.type) {
        case 'add':
        case 'remove':
            return {
                ...change,
                node: expandNode(change.node),
            };
        case 'edit':
            return {
                type: 'edit',
                oldRow: change.oldNode.row,
                row: change.node.row,
            };
        case 'child':
            return {
                type: 'child',
                row: change.node.row,
                child: {
                    ...change.child,
                    change: expandChange(change.child.change),
                },
            };
        default:
            unreachable(change);
    }
}
export function expandNode(node) {
    return {
        row: node.row,
        relationships: Object.fromEntries(Object.entries(node.relationships).map(([k, v]) => [
            k,
            [...v()].map(expandNode),
        ])),
    };
}
//# sourceMappingURL=catch.js.map