import type { AST, Condition, Parameter, SimpleOperator, System } from '../../../zero-protocol/src/ast.ts';
import type { Schema } from '../../../zero-schema/src/builder/schema-builder.ts';
import { type BuilderDelegate } from '../builder/builder.ts';
import type { Format, ViewFactory } from '../ivm/view.ts';
import { type ExpressionFactory } from './expression.ts';
import { type GetFilterType, type HumanReadable, type PreloadOptions, type PullRow, type Query, type RunOptions } from './query.ts';
import { type TTL } from './ttl.ts';
import type { TypedView } from './typed-view.ts';
type AnyQuery = Query<Schema, string, any>;
declare const astSymbol: unique symbol;
export declare function ast(query: Query<Schema, string, any>): AST;
export declare function newQuery<TSchema extends Schema, TTable extends keyof TSchema['tables'] & string>(delegate: QueryDelegate, schema: TSchema, table: TTable): Query<TSchema, TTable>;
export type CommitListener = () => void;
export type GotCallback = (got: boolean) => void;
export interface NewQueryDelegate {
    newQuery<TSchema extends Schema, TTable extends keyof TSchema['tables'] & string, TReturn>(schema: TSchema, table: TTable, ast: AST, format: Format): Query<TSchema, TTable, TReturn>;
}
export interface QueryDelegate extends BuilderDelegate {
    addServerQuery(ast: AST, ttl: TTL, gotCallback?: GotCallback | undefined): () => void;
    updateServerQuery(ast: AST, ttl: TTL): void;
    onTransactionCommit(cb: CommitListener): () => void;
    batchViewUpdates<T>(applyViewUpdates: () => T): T;
    onQueryMaterialized(hash: string, ast: AST, duration: number): void;
    /**
     * Asserts that the `RunOptions` provided to the `run` method are supported in
     * this context. For example, in a custom mutator, the `{type: 'complete'}`
     * option is not supported and this will throw.
     */
    assertValidRunOptions(options?: RunOptions): void;
    /**
     * Client queries start off as false (`unknown`) and are set to true when the
     * server sends the gotQueries message.
     *
     * For things like ZQLite the default is true (aka `complete`) because the
     * data is always available.
     */
    readonly defaultQueryComplete: boolean;
}
export declare function staticParam(anchorClass: 'authData' | 'preMutationRow', field: string | string[]): Parameter;
export declare const SUBQ_PREFIX = "zsubq_";
export declare const defaultFormat: {
    readonly singular: false;
    readonly relationships: {};
};
export declare const newQuerySymbol: unique symbol;
export declare abstract class AbstractQuery<TSchema extends Schema, TTable extends keyof TSchema['tables'] & string, TReturn = PullRow<TTable, TSchema>> implements Query<TSchema, TTable, TReturn> {
    #private;
    readonly format: Format;
    constructor(schema: TSchema, tableName: TTable, ast: AST, format: Format, system: System);
    get [astSymbol](): AST;
    hash(): string;
    protected abstract [newQuerySymbol]<TSchema extends Schema, TTable extends keyof TSchema['tables'] & string, TReturn>(schema: TSchema, table: TTable, ast: AST, format: Format): AbstractQuery<TSchema, TTable, TReturn>;
    one: () => Query<TSchema, TTable, TReturn | undefined>;
    whereExists: (relationship: string, cb?: (q: AnyQuery) => AnyQuery) => Query<TSchema, TTable, TReturn>;
    related: (relationship: string, cb?: (q: AnyQuery) => AnyQuery) => AnyQuery;
    where: (fieldOrExpressionFactory: string | ExpressionFactory<TSchema, TTable>, opOrValue?: SimpleOperator | GetFilterType<any, any, any> | Parameter, value?: GetFilterType<any, any, any> | Parameter) => Query<TSchema, TTable, TReturn>;
    start: (row: Partial<PullRow<TTable, TSchema>>, opts?: {
        inclusive: boolean;
    } | undefined) => Query<TSchema, TTable, TReturn>;
    limit: (limit: number) => Query<TSchema, TTable, TReturn>;
    orderBy: <TSelector extends keyof TSchema["tables"][TTable]["columns"]>(field: TSelector, direction: "asc" | "desc") => Query<TSchema, TTable, TReturn>;
    protected _exists: (relationship: string, cb?: (query: AnyQuery) => AnyQuery) => Condition;
    protected _completeAst(): AST;
    then<TResult1 = HumanReadable<TReturn>, TResult2 = never>(onFulfilled?: ((value: HumanReadable<TReturn>) => TResult1 | PromiseLike<TResult1>) | undefined | null, onRejected?: ((reason: any) => TResult2 | PromiseLike<TResult2>) | undefined | null): PromiseLike<TResult1 | TResult2>;
    abstract materialize(): TypedView<HumanReadable<TReturn>>;
    abstract materialize<T>(factory: ViewFactory<TSchema, TTable, TReturn, T>): T;
    abstract run(options?: RunOptions): Promise<HumanReadable<TReturn>>;
    abstract preload(): {
        cleanup: () => void;
        complete: Promise<void>;
    };
}
declare const completedAstSymbol: unique symbol;
export declare function completedAST(q: Query<Schema, string, any>): AST;
export declare class QueryImpl<TSchema extends Schema, TTable extends keyof TSchema['tables'] & string, TReturn = PullRow<TTable, TSchema>> extends AbstractQuery<TSchema, TTable, TReturn> {
    #private;
    constructor(delegate: QueryDelegate, schema: TSchema, tableName: TTable, ast: AST, format: Format, system?: System);
    get [completedAstSymbol](): AST;
    protected [newQuerySymbol]<TSchema extends Schema, TTable extends string, TReturn>(schema: TSchema, tableName: TTable, ast: AST, format: Format): QueryImpl<TSchema, TTable, TReturn>;
    materialize<T>(factoryOrTTL?: ViewFactory<TSchema, TTable, TReturn, T> | TTL, ttl?: TTL): T;
    run(options?: RunOptions): Promise<HumanReadable<TReturn>>;
    preload(options?: PreloadOptions): {
        cleanup: () => void;
        complete: Promise<void>;
    };
}
export {};
//# sourceMappingURL=query-impl.d.ts.map