import { ExpressionBuilder } from "./expression.js";
import { AbstractQuery, defaultFormat, newQuerySymbol } from "./query-impl.js";
export function staticQuery(schema, tableName) {
    return new StaticQuery(schema, tableName, { table: tableName }, defaultFormat);
}
/**
 * A query that cannot be run.
 * Only serves to generate ASTs.
 */
export class StaticQuery extends AbstractQuery {
    expressionBuilder() {
        return new ExpressionBuilder(this._exists);
    }
    constructor(schema, tableName, ast, format) {
        super(schema, tableName, ast, format, 'permissions');
    }
    [newQuerySymbol](schema, tableName, ast, format) {
        return new StaticQuery(schema, tableName, ast, format);
    }
    get ast() {
        return this._completeAst();
    }
    materialize() {
        throw new Error('StaticQuery cannot be materialized');
    }
    run() {
        return Promise.reject(new Error('StaticQuery cannot be run'));
    }
    preload() {
        throw new Error('StaticQuery cannot be preloaded');
    }
}
//# sourceMappingURL=static-query.js.map