export type TimeUnit = 's' | 'm' | 'h' | 'd' | 'y';
/**
 * Time To Live. This is used for query expiration.
 * - `forever` means the query will never expire.
 * - `none` means the query will expire immediately.
 * - A number means the query will expire after that many milliseconds.
 * - A negative number means the query will never expire, this is same as 'forever'.
 * - A string like `1s` means the query will expire after that many seconds.
 * - A string like `1m` means the query will expire after that many minutes.
 * - A string like `1h` means the query will expire after that many hours.
 * - A string like `1d` means the query will expire after that many days.
 * - A string like `1y` means the query will expire after that many years.
 */
export type TTL = `${number}${TimeUnit}` | 'forever' | 'none' | number;
export declare const DEFAULT_TTL: TTL;
export declare function parseTTL(ttl: TTL): number;
export declare function compareTTL(a: TTL, b: TTL): number;
export declare function normalizeTTL(ttl: TTL): TTL;
//# sourceMappingURL=ttl.d.ts.map