export const DEFAULT_TTL = 'none';
const multiplier = {
    s: 1000,
    m: 60 * 1000,
    h: 60 * 60 * 1000,
    d: 24 * 60 * 60 * 1000,
    y: 365 * 24 * 60 * 60 * 1000,
};
export function parseTTL(ttl) {
    if (typeof ttl === 'number') {
        return Number.isNaN(ttl) ? 0 : !Number.isFinite(ttl) || ttl < 0 ? -1 : ttl;
    }
    if (ttl === 'none') {
        return 0;
    }
    if (ttl === 'forever') {
        return -1;
    }
    const multi = multiplier[ttl[ttl.length - 1]];
    return Number(ttl.slice(0, -1)) * multi;
}
export function compareTTL(a, b) {
    const ap = parseTTL(a);
    const bp = parseTTL(b);
    if (ap === -1 && bp !== -1) {
        return 1;
    }
    if (ap !== -1 && bp === -1) {
        return -1;
    }
    return ap - bp;
}
export function normalizeTTL(ttl) {
    if (typeof ttl === 'string') {
        return ttl;
    }
    if (ttl < 0) {
        return 'forever';
    }
    if (ttl === 0) {
        return 'none';
    }
    let shortest = ttl.toString();
    const lengthOfNumber = shortest.length;
    for (const unit of ['y', 'd', 'h', 'm', 's']) {
        const multi = multiplier[unit];
        const value = ttl / multi;
        const candidate = `${value}${unit}`;
        if (candidate.length < shortest.length) {
            shortest = candidate;
        }
    }
    return (shortest.length < lengthOfNumber ? shortest : ttl);
}
//# sourceMappingURL=ttl.js.map